import sys
import os
import collections

import yaml
import click

from .upload_video import get_authenticated_service, do_upload
from .dumper import yaml_dump


@click.command()
@click.option(
    "--client-secrets",
    metavar="<client_secrets_json_file>",
    show_default=True,
    type=click.Path(dir_okay=False),
    default=os.path.join(sys.prefix, "share", "talk-video-uploader",
                         "client_id.json"),
    help="Path to OAuth2 client secret JSON file. "
    "Only needed when generating new credentials.")
@click.option(
    "--credentials",
    metavar="<oauth2_credentials_json_file>",
    show_default=True,
    type=click.Path(dir_okay=False),
    default=os.path.join(
        click.get_app_dir("talk-video-uploader"),
        "youtube_credentials.json"
    ),
    help="Path to OAuth2 credentials JSON file. "
         "Will be generated if necessary.")
@click.argument(
    "files",
    nargs=-1,
    type=click.Path(exists=True, dir_okay=False),
    metavar="<metadata_yaml_file>…")
def main(client_secrets, credentials, files):
    """
    Upload Pyvo videos with proper metadata.

    Client secrets are bundled but can be also obtained from Google for free.
    Credentials file will be generated if non-existent.
    """
    youtube = get_authenticated_service(client_secrets, credentials)
    pyvometa = collections.defaultdict(list)
    for f in files:
        fbase, fext = os.path.splitext(f)
        if fext.lower() not in [".yaml", ".yml"]:
            click.echo(click.style("Ignoring file {} with unknown "
                       "extension.".format(f), fg="red"))
            continue
        try:
            with open(f) as inf:
                meta = yaml.safe_load(inf)
        except FileNotFoundError:
            click.echo(click.style("Metadata file {} not found!".format(f),
                                   fg="red"))
            continue
        videofile = meta.get("fname")
        if videofile:
            videofile = os.path.join(os.path.dirname(f), videofile)
        else:
            videofile = fbase + ".mkv"
        if not os.path.exists(videofile):
            click.echo(click.style("Video file {} "
                                   "not found!".format(videofile), fg="red"))
            continue

        tags = meta.get("tags")
        if not tags:
            tags = ["Python", "Pyvo"]
        if meta.get("lightning"):
            meta["lt"] = "\N{HIGH VOLTAGE SIGN} "
            tags.append("Lightning talk")
        else:
            meta["lt"] = ""

        youtube_body = {
            "snippet": {
                "title": "{lt}{speaker} – {title}".format_map(meta),
                "description": "{event} – {date}\n{url}".format_map(meta),
                "tags": tags,
                "categoryId": 27,  # Education
                },
            "status": {
                "privacyStatus": "unlisted",
                },
            "recordingDetails": {
                "recordingDate": "{date}T18:00:00.000Z".format_map(meta),
                },
        }

        lang = meta.get("language")
        if lang:
            if "audio" in lang:
                youtube_body["snippet"]["defaultAudioLanguage"] = lang["audio"]

        video_url = do_upload(youtube, videofile, youtube_body)
        if video_url:
            talkmeta = collections.OrderedDict()
            talkmeta["title"] = meta["title"]
            talkmeta["speakers"] = meta["speaker"].split(", ")
            if meta.get("lightning"):
                talkmeta["lightning"] = True
            if lang:
                talkmeta["language"] = lang
            talkmeta["coverage"] = [{"video": video_url}, ]
            talkyaml = yaml_dump([talkmeta, ])
            click.echo(talkyaml)
            pyvometa[meta["url"]].append(talkmeta)

    for url, talkmeta in pyvometa.items():
        click.echo(click.style(url, bold=True))
        click.echo("-"*len(url))
        click.echo(yaml_dump({"talks": talkmeta}))


if __name__ == "__main__":
    # pylint doesn't realize that click has changed the function signature.
    main()  # pylint: disable=no-value-for-parameter




import http.client
import httplib2
import random
import time
import json
import os
import sys

from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload
from google_auth_oauthlib.flow import InstalledAppFlow
from google.oauth2.credentials import Credentials

import click

# Explicitly tell the underlying HTTP transport library not to retry, since
# we are handling retry logic ourselves.
httplib2.RETRIES = 1

# Maximum number of times to retry before giving up.
MAX_RETRIES = 10

# Always retry when these exceptions are raised.
RETRIABLE_EXCEPTIONS = (httplib2.HttpLib2Error, IOError,
                        http.client.NotConnected,
                        http.client.IncompleteRead,
                        http.client.ImproperConnectionState,
                        http.client.CannotSendRequest,
                        http.client.CannotSendHeader,
                        http.client.ResponseNotReady,
                        http.client.BadStatusLine)

# Always retry when an apiclient.errors.HttpError with one of these status
# codes is raised.
RETRIABLE_STATUS_CODES = [500, 502, 503, 504]

# This OAuth 2.0 access scope allows an application to upload files to the
# authenticated user's YouTube channel, but doesn't allow
# other types of access.
SCOPES = ["https://www.googleapis.com/auth/youtube.upload"]
API_SERVICE_NAME = "youtube"
API_VERSION = "v3"


def save_credentials(creds, filename):
    creds_data = {
        "refresh_token": creds.refresh_token,
        "token_uri": creds.token_uri,
        "client_id": creds.client_id,
        "client_secret": creds.client_secret,
        "scopes": creds.scopes
    }
    try:
        os.makedirs(os.path.dirname(filename))
    except FileExistsError:
        pass
    with open(filename, "w") as outfile:
        json.dump(creds_data, outfile)


def get_authenticated_service(client_secrets, creds_file, local_server=False):
    """
    Either load or create authorization credentials
    and return service object.
    """
    try:
        credentials = Credentials.from_authorized_user_file(creds_file)
    except (ValueError, FileNotFoundError):
        credentials = None
    if not credentials:
        if not os.path.isfile(client_secrets):
            click.echo(click.style("Missing client secrets file {}, "
                                   "cannot proceed.".format(client_secrets),
                                   fg="red"))
            sys.exit(1)
        flow = InstalledAppFlow.from_client_secrets_file(client_secrets,
                                                         SCOPES)
        if local_server:
            credentials = flow.run_local_server()
        else:
            credentials = flow.run_console()
        save_credentials(credentials, creds_file)
    return build(API_SERVICE_NAME, API_VERSION, credentials=credentials)


def do_upload(youtube, videofile, body):

    # Call the API's videos.insert method to create and upload the video.
    request = youtube.videos().insert(
        part=",".join(list(body.keys())),
        body=body,
        media_body=MediaFileUpload(videofile, chunksize=10*1024*1024,
                                   resumable=True)
    )

    response = None
    error = None
    retry = 0
    click.echo("".join([click.style("Uploading file ", bold=True),
                        os.path.basename(videofile),
                        "…"]))
    with click.progressbar(length=request.resumable.size()) as progress:
        while response is None:
            try:
                status, response = request.next_chunk()
                if status:
                    progress.update(status.resumable_progress)
                elif response:
                    progress.update(request.resumable.size())
            except HttpError as e:
                if e.resp.status in RETRIABLE_STATUS_CODES:
                    error = "A retriable HTTP error {} occurred:\n{}".format(
                            e.resp.status, e.content)
                else:
                    raise
            except RETRIABLE_EXCEPTIONS as e:
                error = "A retriable error occurred: {}".format(e)

            if error:
                click.echo()
                click.echo(click.style(error, fg="red"))
                retry += 1
                if retry > MAX_RETRIES:
                    click.echo(click.style("No longer attempting to retry.",
                                           fg="red"))
                    return None

                max_sleep = 2 ** retry
                sleep_seconds = random.random() * max_sleep
                click.echo("Sleeping {} seconds and then retrying…".format(
                           sleep_seconds))
                time.sleep(sleep_seconds)
                error = None

    if "id" in response:
        click.echo(click.style("Video was successfully uploaded.", fg="green",
                               bold=True))
        video_url = "https://www.youtube.com/watch?v={}".format(response["id"])
        return video_url
    else:
        click.echo(click.style("The upload failed with an unexpected "
                               "response: {}".format(response),
                               fg="red"))
        return None



import collections

import yaml


def yaml_dump(data):
    return yaml.dump(data, Dumper=EventDumper)


def _dict_representer(dumper, data):
    return dumper.represent_mapping(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
        data.items())


class EventDumper(yaml.SafeDumper):
    def __init__(self, *args, **kwargs):
        kwargs["default_flow_style"] = False
        kwargs["allow_unicode"] = True
        super(EventDumper, self).__init__(*args, **kwargs)


EventDumper.add_representer(collections.OrderedDict, _dict_representer)



from setuptools import setup
from pathlib import Path

readme = Path(__file__).with_name("README.rst").read_text()

setup(
    name='talk_video_uploader',
    version='0.1',
    description='Batch upload talk videos to YouTube',
    long_description=readme,
    long_description_content_type="text/x-rst",
    url='https://github.com/oskar456/talk-video-uploader',
    author='Ondřej Caletka',
    author_email='ondrej@caletka.cz',
    license='MIT',
    python_requires=">=3.6",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Multimedia :: Video',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: 3 :: Only",
    ],
    packages=['talk_video_uploader'],
    data_files=[('share/talk-video-uploader', ['client_id.json'])],
    install_requires=[
        'google-api-python-client',
        'google-auth-httplib2',
        'google-auth-oauthlib',
        'pyyaml',
        'click'
    ],
    entry_points={
        'console_scripts': [
            'talk-video-uploader = talk_video_uploader.__main__:main',
        ],
    }
)
